#ifndef COMMON_H
#define COMMON_H


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdbool.h>

#include <pspaudio.h>
#include <psputility.h>
#include <pspdebug.h>
#include <pspkernel.h>
#include <pspsdk.h>
#include <pspctrl.h>
#include <pspdisplay.h>
#include <pspdisplay_kernel.h>
#include <pspgu.h>
#include <psptypes.h>
#include <pspiofilemgr.h>
#include <pspsysmem_kernel.h>
#include <pspmscm.h>

#include "memory.h"
#include "io.h"
#include "unicode.h"
#include "libmenu.h"
#include "message.h"

#define MAX_DISPLAY_NUM 26
#define ERR (0)

#define SCREEN_WIDTH 480
#define SCREEN_HEIGHT 272

#define	PSP_LINE_SIZE 512
#define BUF_WIDTH (512)
#define SCR_WIDTH (480)
#define SCR_HEIGHT (272)
#define PIXEL_SIZE (2) /* change this if you change to another screenmode */
#define FRAMEBUFFER_SIZE (BUF_WIDTH * SCR_HEIGHT * PIXEL_SIZE)
#define ZBUF_SIZE (BUF_WIDTH SCR_HEIGHT * 2) /* zbuffer seems to be 16-bit? */


#define printf_j(x, y, format, ... ) libmPrintfXY(x, y, BLACK, 0, vram_addr, 512, format, ##__VA_ARGS__)
#define libmStrLen(str) libmPrintXY(0, 0, 0, 0, str)
#define libmStrMeasure(str) libmStrLen(str) * LIBM_CHAR_WIDTH
#define libmClearScreen(color) libmFillRect(0, 0, 480, 272, color)
#define libmFillRectEx(color, sx, sy, width, height) libmFillRect(sx, sy, sx+width, sy+height, color)

#define LIST_CHAR_HEIGHT (LIBM_CHAR_HEIGHT+1)

#define TITLE_POS 6
#define LIST_POS 20
#define DETAIL_POS 260


enum colors {
		RED = 0xFF0000FF,
		GREEN =	0xFF00FF00,
		BLUE = 0xFFFF0000,
		WHITE =	0xFFFFFFFF,
		LITEGRAY = 0xFFBFBFBF,
		GRAY = 0xFF7F7F7F,
		DARKGRAY = 0xFF3F3F3F,		
		BLACK = 0xFF000000,
		BROWN = 0xFF404080,
		PINK = 0xFFFF80FF,
		YELLOWGREEN = 0xFF00FF80,
		LIGHTBLUE = 0xFFD8EFA0,
		PURPLE = 0xFFFF0080,
		ORANGE = 0xFF4080FF,
		YELLOW = 0xFF00FFFF,
		SILVER = 0xFFC0C0C0,
		GOLD = 0xFF20C0E6,
};

extern const char *UNIT[];
extern int MSinserted;
extern void *vram_addr;

typedef struct CONV_ {
	float size;
	int unit;
} CONV;

int SetupCallbacks(void);

void qsort(void *base, unsigned num, unsigned width, int (*comp)(const void *, const void *));
void wait_button_up(SceCtrlData *data);
void get_button(SceCtrlData *data);
void print_message(char *message);
void initGraphics();
int OnScreenKeybord(char *buf, int buflen, int mode, const char *title, const char *init);
void conv_unit(u64 src, CONV *conv_list);
int checkYesNo(char *message);
int checkMS(char *message, int *check);
int checkSameMS(int *check);
bool isInFolder(const char *dir, const char *file);


void *GetVramAdder(int size, int align);
void FreeVramAdder(void *ptr);


#endif